@@ -1,3 +1,25 @@
+# (The MIT License)
+#
+# Copyright (c) 2013 Kura
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the 'Software'), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 """blackhole.connection - Provide mechanisms for processing socket data.
 
 This module provides methods for Blackhole to use internally
@@ -10,6 +32,7 @@ import ssl
 import sys
 
 from tornado import iostream
+from tornado.ioloop import IOLoop
 from tornado.options import options
 
 from blackhole import __fullname__
@@ -65,25 +88,21 @@ def connection_stream(connection):
     from stdlib.
     """
     if connection.getsockname()[1] == options.ssl_port and options.ssl:
-        try:
-            ssl_connection = ssl.wrap_socket(connection, **sslkwargs)
-        except (ssl.SSLError, socket.error) as e:
-            if e.errno == ssl.SSL_ERROR_EOF or e.errno == errno.ECONNABORTED:
-                ssl_connection.close()
-                return
-            else:
-                raise
-        # Do a nasty blanket Exception until SSL exceptions are fully known
-        try:
-            return iostream.SSLIOStream(ssl_connection)
-        except Exception as e:
-            log.error(e)
-            ssl_connection.close()
-            return
+        return ssl_connection(connection)
     else:
         return iostream.IOStream(connection)
 
 
+def ssl_connection(connection):
+    try:
+        ssl_connection = ssl.wrap_socket(connection, **sslkwargs)
+        return iostream.SSLIOStream(ssl_connection)
+    except (ssl.SSLError, socket.error) as e:
+        if e.errno == ssl.SSL_ERROR_EOF or e.errno == errno.ECONNABORTED:
+            ssl_connection.close()
+            return
+
+
 def handle_command(line, mail_state):
     """Handle each SMTP command as it's sent to the server
 
@@ -99,9 +118,6 @@ def handle_command(line, mail_state):
         if line[0] == "." and len(line) == 3 and ord(line[0]) == 46:
             mail_state.reading = False
             resp = response()
-            from tornado.ioloop import IOLoop
-            import time
-            IOLoop.instance().add_timeout(time.time() + 5, time.time())
     elif line.lower().startswith("ehlo"):
         resp = ["%s\r\n" % x for x in EHLO_RESPONSES]
     elif any(line.lower().startswith(e) for e in ['helo', 'mail from',
@@ -127,7 +143,7 @@ def handle_command(line, mail_state):
         mail_state.reading = True
     else:
         resp = response(500)
-    
+
     # this is a blocking action, sadly
     # async non blocking methods did not
     # work. =(
@@ -139,10 +155,10 @@ def handle_command(line, mail_state):
     return resp, close
 
 
-def write_response(stream, mail_state, resp):
+def write_response(mail_state, resp):
     """Write the response back to the stream"""
     log.debug("[%s] SEND: %s" % (mail_state.email_id, resp.upper().rstrip()))
-    stream.write(resp)
+    mail_state.stream.write(resp)
 
 
 def connection_ready(sock, fd, events):
@@ -170,6 +186,7 @@ def connection_ready(sock, fd, events):
             return
         mail_state = MailState()
         mail_state.email_id = email_id()
+        mail_state.stream = stream
 
         # Sadly there is nothing I can do about the handle and loop
         # fuctions. They have to exist within connection_ready
@@ -183,16 +200,19 @@ def connection_ready(sock, fd, events):
             resp, close = handle_command(line, mail_state)
             if resp:
                 if isinstance(resp, list):
-                    # This is required for returning
-                    # multiple status codes for EHLO
                     for r in resp:
-                        write_response(stream, mail_state, r)
+                        write_response(mail_state, r)
                 else:
                     # Otherwise it's a single response
-                    write_response(stream, mail_state, resp)
+                    write_response(mail_state, resp)
+            if line.lower().startswith("starttls"):
+                fileno = mail_state.stream.socket.fileno()
+                IOLoop.current().remove_handler(fileno)
+                mail_state.stream = ssl_connection(connection)
             if close is True:
                 log.debug("Closing")
-                stream.close()
+                mail_state.stream.close()
+                del mail_state.stream
                 return
             else:
                 loop()
@@ -202,9 +222,10 @@ def connection_ready(sock, fd, events):
             Loop over the socket data until we receive
             a newline character (\n)
             """
-            stream.read_until("\n", handle)
+            # Protection against stream already reading exceptions
+            if not mail_state.stream.reading():
+                mail_state.stream.read_until("\n", handle)
 
-        hm = "220 %s [%s]\r\n" % (get_mailname(),
-                              __fullname__)
-        stream.write(hm)
+        hm = "220 %s [%s]\r\n" % (get_mailname(), __fullname__)
+        mail_state.stream.write(hm)
         loop()
